#import "@preview/cetz:0.2.2": canvas, draw
#import "renderer.typ": *

#let anchor-line(
  name,
  anchors
) = {
  return ((
    type: "anchors",
    name: name,
    inset: 1em,
    anchors: anchors,
  ),)
}

#let sign-line(
  name,
  values
) = {
  return ((
    type: "sign",
    name: name,
    inset: 1em,
    values: values,
  ),)
}

#let func-line(
  name,
  values,
  mult: 1
) = {
  return ((
    type: "func",
    name: name,
    inset: (
      top: 1em * mult,
      bottom: 1em * mult,
      left: 1em,
      right: 1em
    ),
    values: values,
  ),)
}

#let convex-line(
  name,
  values,
  mult: 1
) = {
  return ((
    type: "convex",
    name: name,
    inset: (
      top: 1em * mult,
      bottom: 1em * mult,
      left: 1em,
      right: 1em
    ),
    values: values,
  ),)
}

#let sign-table(
  anchors,
  lines,
  var: $x$,
  espcl: 1,
  deltacl: 1,
) = context {
  let lines = anchor-line(var, anchors) + lines
  let name-sizes = lines.map(l => measure(box(l.name, inset: l.inset)))
  let headers-width = calc.max(..name-sizes.map(s => s.width))
  let headers-cx = headers-width / 2
  let styles = (
    espcl: espcl,
    deltacl: deltacl
  )

  let render-anchors = render-anchors.with(styles)
  let render-signs = render-signs.with(styles)
  let render-func = render-func.with(styles)
  let render-convex = render-convex.with(styles)

  canvas({
    /*draw.group(
      name: "headers",
      {
      }
    )*/
    for (i, line) in lines.enumerate() {
      let pos = if i == 0 {
        (headers-cx, 0)
      } else {
        "header-" + str(i - 1) + ".south"
      }
      let anchor-name = "header-" + str(i)
      draw.content(
        name: anchor-name,
        anchor: "north",
        padding: line.inset,
        pos,
        line.name
      )
      draw.rect(
        name: anchor-name,
        (v => (0, v.at(1), 0), anchor-name + ".north-west"),
        (v => (headers-width, v.at(1), 0), anchor-name + ".south-east"),
        stroke: black + .5pt
      )
    }
    for (i, line) in lines.enumerate() {
      draw.get-ctx(ctx => {
        if line.type == "anchors" {
          render-anchors(ctx, line)
        } else if line.type == "sign" {
          render-signs(ctx, line, i)
        } else if line.type == "func" {
          render-func(ctx, line, i)
        } else if line.type == "convex" {
          render-convex(ctx, line, i)
        }
      })
    }

    draw.line(
      "anchors.south-east",
      (
        horizontal: (),
        vertical: "header-" + str(lines.len() - 1) + ".south-east"
      ),
      stroke: black + .5pt
    )
  })
}